/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: CarDeployableMetadataFactory.java 3243 2008-05-15 14:39:38Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.car.impl;

import java.util.List;

import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.deploy.api.deployable.CARDeployable;
import org.ow2.util.ee.deploy.api.deployable.metadata.DeployableMetadataException;
import org.ow2.util.ee.metadata.car.api.ICarDeployableMetadata;
import org.ow2.util.ee.metadata.car.api.ICarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.car.impl.configurator.CarDeployableMetadataConfigurator;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.ArchiveScanner;
import org.ow2.util.scan.api.ArchiveScannerFactory;
import org.ow2.util.scan.api.IArchiveScannerFactory;
import org.ow2.util.scan.api.ScanException;

/**
 * Factory to contruct war deployable metadata.
 * @author Gael Lalire
 */
public class CarDeployableMetadataFactory implements ICarDeployableMetadataFactory {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(CarDeployableMetadataFactory.class);

    /**
     * Archive scanner factory.
     */
    private IArchiveScannerFactory archiveScannerFactory;

    /**
     * Constructor.
     * @throws ScanException if no archive scanner factory can be found
     */
    public CarDeployableMetadataFactory() throws ScanException {
        this(ArchiveScannerFactory.getInstance());
    }

    /**
     * Constructor with an archive scanner factory.
     * @param archiveScannerFactory the archive scanner factory
     */
    public CarDeployableMetadataFactory(final IArchiveScannerFactory archiveScannerFactory) {
        this.archiveScannerFactory = archiveScannerFactory;
    }

    /**
     * Create war deployable metadata.
     * @param deployable the war deployable
     * @return the metadata
     * @throws DeployableMetadataException if error occurs
     */
    public ICarDeployableMetadata createDeployableMetadata(final CARDeployable deployable) throws DeployableMetadataException {
        return createDeployableMetadata(deployable, null);
    }

    /**
     * Create war deployable metadata.
     * @param deployable the war deployable
     * @param carExternLibraries extern libraries
     * @return the metadata
     * @throws DeployableMetadataException if error occurs
     */
    public ICarDeployableMetadata createDeployableMetadata(final CARDeployable deployable,
            final CarExternLibraries carExternLibraries) throws DeployableMetadataException {

        IArchive archive = deployable.getArchive();

        long tAnalyzeStart = System.currentTimeMillis();

        ArchiveScanner archiveScanner = this.archiveScannerFactory.createArchiveScanner(archive);

        CarDeployableMetadataConfigurator carArchiveMetadataConfigurator = new CarDeployableMetadataConfigurator(deployable);


        archiveScanner.addArchiveConfigurator(carArchiveMetadataConfigurator);

        try {
            archiveScanner.scanAll();
            // TODO gaellalire : search unknow parent classes in extern libraries
        } catch (ScanException e) {
            throw new DeployableMetadataException(e);
        }

        // time if debugging
        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("Analyze of file {0} took {1} ms.", archive.getName(), (tAnalyzeStartEnd - tAnalyzeStart));
            }
        }

        return carArchiveMetadataConfigurator.getCarDeployableMetadata();
    }

    /**
     * Add a given class in the list only if it is not yet present in the list.
     * @param className the given name of the class
     * @param classNames the lsit of existing classes.
     */
    protected void addClassIfNotAlreadyPresent(final String className, final List<String> classNames) {
        // Sometimes, servlet classname is null as it is referenced as a
        // jsp-file
        if (className == null) {
            return;
        }
        if (!classNames.contains(className)) {
            classNames.add(className);
        }
    }

    /**
     * @return the deployable metadata class
     */
    public Class<ICarDeployableMetadata> getDeployableMetadataClass() {
        return ICarDeployableMetadata.class;
    }

}
