/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: CarExternLibraries.java 3253 2008-05-16 12:47:44Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.car.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.metadata.car.api.ICarClassMetadata;
import org.ow2.util.ee.metadata.car.api.ICarDeployableMetadata;
import org.ow2.util.ee.metadata.car.impl.configurator.CarDeployableMetadataConfigurator;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.ArchiveScanner;
import org.ow2.util.scan.api.ArchiveScannerFactory;
import org.ow2.util.scan.api.ScanException;

/**
 * Default implementation.
 * @author Gael Lalire
 */
public class CarExternLibraries {

    /**
     * A logger.
     */
    private static final Log LOGGER = LogFactory.getLog(CarExternLibraries.class);

    /**
     * Associate a class name to its metadata.
     */
    private Map<String, ICarClassMetadata> classMetadata;

    /**
     * Associate an archive scanner to its metadata.
     */
    private Map<ArchiveScanner, ICarDeployableMetadata> scanners;

    /**
     * Construct an extern library.
     * @param archiveScannerFactory the archive scanner factory
     * @param archiveList the archive list
     */
    public CarExternLibraries(final ArchiveScannerFactory archiveScannerFactory, final List<IArchive> archiveList) {
        classMetadata = new HashMap<String, ICarClassMetadata>();
        scanners = new HashMap<ArchiveScanner, ICarDeployableMetadata>();
        for (IArchive archive : archiveList) {
            // no lib deployable so call getDeployable will return null
            CarDeployableMetadataConfigurator warArchiveMetadataConfigurator =
                new CarDeployableMetadataConfigurator(null);
            ArchiveScanner archiveScanner = archiveScannerFactory.createArchiveScanner(archive);
            archiveScanner.addArchiveConfigurator(warArchiveMetadataConfigurator);
            scanners.put(archiveScanner, warArchiveMetadataConfigurator.getCarDeployableMetadata());
        }
    }

    /**
     * Try to find extern metadata.
     * @param className the class name
     * @return the metadata or null
     */
    public ICarClassMetadata findMetadata(final String className) {
        ICarClassMetadata warClassMetadata = classMetadata.get(className);
        if (warClassMetadata != null) {
            return warClassMetadata;
        }
        List<String> classNameList = new ArrayList<String>();
        classNameList.add(className);
        Iterator<Entry<ArchiveScanner, ICarDeployableMetadata>> entries = scanners.entrySet().iterator();
        while (warClassMetadata == null && entries.hasNext()) {
            Entry<ArchiveScanner, ICarDeployableMetadata> entry = entries.next();
            try {
                // TODO gaellalire : add method scanClassName (instead of scan collection)
                // which returns a boolean
                entry.getKey().scanCollection(classNameList);
                warClassMetadata = entry.getValue().getCarClassMetadata(className);
            } catch (ScanException e) {
                LOGGER.debug("error when scan collection", e);
            }
        }
        if (warClassMetadata != null) {
            classMetadata.put(className, warClassMetadata);
        }
        return warClassMetadata;
    }

    /**
     * Stop all archive scanner.
     */
    public void close() {
        for (ArchiveScanner archiveScanner : scanners.keySet()) {
            archiveScanner.endScan();
        }
    }

}
